# tgx_classifier.R
#
# CMD: Rscript tgx_classifier.R [DDI|HDACi] [Data File] [Unique ID] [Info File]
# e.g.
# $ Rscript tgx_classifier.R HDACi "One-Chem_Multi-Conc_log2_Norm_Data.tsv" 123 "One-Chem_Multi-Conc_log2_Norm_Metadata.tsv"
# $ Rscript tgx_classifier.R DDI "One-Chem_Multi-Conc_log2_Norm_Data.tsv.tsv" 456 "One-Chem_Multi-Conc_log2_Norm_Metadata.tsv"

# Get script base path
args <- commandArgs();
script_info <- strsplit(args[4], "=")[[1]]
base_dir <- dirname(normalizePath(script_info[2]))
args <- commandArgs(TRUE);

if (length(args) < 3) {
  stop("Missing cmd arguments, Stop process!")
}

biomarker <- args[1]
infile_name <- args[2]
unique_id <- args[3]
info_file_name <- NULL

if (length(args) > 3) {
  info_file_name <- args[4]
}

print(paste("Input parameters", paste(biomarker, infile_name, unique_id, info_file_name, sep = ", "), sep = ": "))
# END read CMD arguments

options(warn=-1)
# options(scipen=99)  # use this option only if you do NOT want scientific notation in the outputs
# options(stringsAsFactors = FALSE)

source(file.path(base_dir, "source_utils.R")) # with minimum/basic functions
sub_dirs <- get_sub_dirs()
load_shared_libraries()
load_directory_source(file.path(base_dir, sub_dirs$code))
# configs <- read_configs(base_dir)
set_time_zone("EST")
timestamp <- format(Sys.time(), "%Y%m%d%H") # 2021071517
datestamp <-  format(Sys.Date(), "%Y%m%d")
date_stamp <- format(Sys.time(), "%d-%b-%Y")
unique_name <- paste(biomarker, unique_id, sep = "_")
out_path <- file.path(base_dir, sub_dirs$results, unique_name)

if (!validate_f_path(out_path)) {
  stop("Stop process")
}

log_file_name <- paste(paste(get_log_prefix(), datestamp, sep = "_"), "log", sep = ".")
log_file <- file.path(base_dir, sub_dirs$logs, log_file_name)
cat(format(Sys.time(), "# %a %b %d %X %Y"), infile_name, file = log_file, sep = ", ", fill = TRUE, append = TRUE);

error_file_name <- paste(biomarker, unique_id, get_error_file_suffix(), sep = "_")
error_file <- file.path(base_dir, sub_dirs$logs, error_file_name)

# Classifier files: gene signature file and heat map file
classifier_files <- get_classifier_files()

file_heat <- file.path(base_dir, sub_dirs$classifier, paste(biomarker, classifier_files$heatmap_dat_file, sep = "_"))
file_clf <- file.path(base_dir, sub_dirs$classifier, paste(biomarker, classifier_files$classifier_file, sep = "_"))
file_new <- file.path(base_dir, sub_dirs$data, infile_name) #"One-Chem_Multi-Conc_log2_Norm_Data.tsv"
files_list <- c(file_heat, file_clf, file_new)

if (!is.null(info_file_name)) {
  file_info <- file.path(base_dir, sub_dirs$data, info_file_name) # column metadata file
  files_list <- c(files_list, file_info)
}

is_valid <- validate_infiles(error_file, files_list)

if (is_valid) {
  data_heat <- read.delim(file_heat, header = TRUE, check.names=FALSE)
  data_clf <- read.delim(file_clf, header = TRUE, check.names=FALSE)
  data_new <- read.delim(file_new, header = TRUE, check.names=FALSE)
  data_info <- NULL

  if (!is.null(info_file_name)) {
    data_info <- read.delim(file_info, header = TRUE, check.names=FALSE)
    names(data_info) <- tolower(names(data_info))
    
    if (FALSE %in% (data_info$column_name %in% names(data_new))) {
      cols_missed <- data_info$column_name[!data_info$column_name %in% names(data_new)]
  
      if (length(cols_missed) > 0) {
        if (length(cols_missed) == 1) {
          cat("The column does not exist in the data file", cols_missed, file = error_file, sep = ": ", fill = TRUE, append = TRUE)
        } else {
          cat("The columns do not exist in the data file", paste(cols_missed, collapse = ", "), file = error_file, sep = ": ", fill = TRUE, append = TRUE)
        }
        
        cat("The job will not be processed due to column mis-match.", file = error_file, fill = TRUE, append = TRUE)
        stop("The job will not be processed due to column mis-match.")
      }
    } else {
      if (ncol(data_new) - nrow(data_info) > 1) {
        cat("It seems that the number of metadata columns is less than number of columns in the data file.", file = log_file, fill = TRUE, append = TRUE)
        data_new <- data_new[, c(names(data_new)[1], data_info$column_name)]
      }
    }
  }

  # Only work for two-column data, column 1:gene symbols, column2: log2 ratio/fold change
  txt_file_labels <- get_txt_file_labels()
  file_extensions <- get_file_extensions()
  results_summary <- NULL
  col <- 2

  for (col in 2:ncol(data_new)) {
    if (is_na_only(data_new[[col]])) {
      break;
    }

    if (is.null(data_info)) {
      name_label <- names(data_new)[col]
      title_plot <- name_label
      cur_info <- match_column_info(name_label)
      # title_info <- parse_column_info(name_label)
      
      # if (!is.null(title_info$does_unit)) {
      #   title_plot <- paste(title_info$name, "at", title_info$does_unit, sep = " ")
      # }
      
      if (cur_info$concentration_unit != "") {
        title_plot <- create_plot_title(cur_info)
      }
    } else {
      cur_info <- data_info[col - 1, ]
      name_label <- as.character(cur_info$column_name[1])
      title_plot <- create_plot_title(cur_info)
    }

    if (biomarker == get_default_biomarkers()$hdaci) {
      classified <- classifier_hdaci(biomarker, data_clf, data_heat, data_new[, c(1, col)], out_path, infile_name, unique_id, name_label, title_plot)
    } else if (biomarker == get_default_biomarkers()$ddi) {
      classified <- classifier_ddi(biomarker, data_clf, data_heat, data_new[, c(1, col)], out_path, infile_name, unique_id, name_label, title_plot)
    } else {
      print(paste("Not handled classifier", biomarker, sep = ": "))
    }

    # Add ("COLUMN_NAME", "CHEMICAL", "SHORT_LABEL", "CONCENTRATION", "CONCENTRATION_UNIT")
    if (!is.null(data_info)) {
      names(cur_info) <- toupper(names(cur_info))
      classified <- cbind(cur_info, classified)
    }
    
    results_summary <- rbind(results_summary, classified)
  }

  results_summary$COLUMN_NAME <- as.character(results_summary$COLUMN_NAME)
  results_summary$Name <- as.character(results_summary$Name)
  results_summary <- results_summary[results_summary$COLUMN_NAME == results_summary$Name, ]
  results_summary$Name <- NULL
  
  file_summary <- write_to_tab_del_file(out_path, unique_name, txt_file_labels$summary, file_extensions$txt, results_summary)
  zip_file_name <- compress_results(biomarker, unique_id, out_path, log_file)
  # Copy summary file from results folder to summary folder
  summary_path <- file.path(base_dir, sub_dirs$summary)
  copy_summary_file(out_path, summary_path, file_summary)
  print(paste("Summary file", file_summary, sep = ": "))
  print(paste("ZIP file", zip_file_name, sep = ": "))
} else {
  stop("Input files failed validation")
}
